# coding: utf-8

"""
ColiVara

Colivara is a suite of services that allows you to store, search, and retrieve documents based on their visual embeddings.      It is a web-first implementation of the ColPali paper using ColQwen2 as backend model. It works exacly like RAG from the end-user standpoint - but using vision models instead of chunking and text-processing for documents.

The version of the OpenAPI document: 1.0.0
Contact: abdulhaleem@tjmlabs.com
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest

from colivara_py.models.document_out import DocumentOut


class TestDocumentOut(unittest.TestCase):
    """DocumentOut unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> DocumentOut:
        """Test DocumentOut
        include_optional is a boolean, when False only required
        params are included, when True both required and
        optional params are included"""
        # uncomment below to create an instance of `DocumentOut`
        """
        model = DocumentOut()
        if include_optional:
            return DocumentOut(
                id = 56,
                name = '',
                metadata = colivara_py.models.metadata.Metadata(),
                url = '',
                num_pages = 56,
                collection_name = '',
                pages = [
                    colivara_py.models.page_out.PageOut(
                        document_name = '', 
                        img_base64 = '', 
                        page_number = 56, )
                    ]
            )
        else:
            return DocumentOut(
                id = 56,
                name = '',
                num_pages = 56,
                collection_name = '',
        )
        """

    def testDocumentOut(self):
        """Test DocumentOut"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
