# coding: utf-8

# flake8: noqa

"""
    ColiVara

    Colivara is a suite of services that allows you to store, search, and retrieve documents based on their visual embeddings.      It is a web-first implementation of the ColPali paper using ColQwen2 as backend model. It works exacly like RAG from the end-user standpoint - but using vision models instead of chunking and text-processing for documents.

    The version of the OpenAPI document: 1.0.0
    Contact: founders@colivara.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501



# import apis into sdk package
from colivara_py.api.collections_api import CollectionsApi
from colivara_py.api.documents_api import DocumentsApi
from colivara_py.api.embeddings_api import EmbeddingsApi
from colivara_py.api.filter_api import FilterApi
from colivara_py.api.health_api import HealthApi
from colivara_py.api.helpers_api import HelpersApi
from colivara_py.api.search_api import SearchApi
from colivara_py.api.webhook_api import WebhookApi

# import ApiClient
from colivara_py.api_response import ApiResponse
from colivara_py.api_client import ApiClient
from colivara_py.configuration import Configuration
from colivara_py.exceptions import OpenApiException
from colivara_py.exceptions import ApiTypeError
from colivara_py.exceptions import ApiValueError
from colivara_py.exceptions import ApiKeyError
from colivara_py.exceptions import ApiAttributeError
from colivara_py.exceptions import ApiException

# import models into sdk package
from colivara_py.models.collection_in import CollectionIn
from colivara_py.models.collection_out import CollectionOut
from colivara_py.models.document_in import DocumentIn
from colivara_py.models.document_in_patch import DocumentInPatch
from colivara_py.models.document_out import DocumentOut
from colivara_py.models.embeddings_in import EmbeddingsIn
from colivara_py.models.embeddings_out import EmbeddingsOut
from colivara_py.models.file_out import FileOut
from colivara_py.models.generic_error import GenericError
from colivara_py.models.generic_message import GenericMessage
from colivara_py.models.key import Key
from colivara_py.models.lookup_enum import LookupEnum
from colivara_py.models.on_enum import OnEnum
from colivara_py.models.page_out import PageOut
from colivara_py.models.page_out_query import PageOutQuery
from colivara_py.models.patch_collection_in import PatchCollectionIn
from colivara_py.models.query_filter import QueryFilter
from colivara_py.models.query_in import QueryIn
from colivara_py.models.query_out import QueryOut
from colivara_py.models.response import Response
from colivara_py.models.search_image_in import SearchImageIn
from colivara_py.models.search_image_out import SearchImageOut
from colivara_py.models.task_enum import TaskEnum
from colivara_py.models.value import Value
from colivara_py.models.webhook_in import WebhookIn
from colivara_py.models.webhook_out import WebhookOut


from colivara_py.client import ColiVara


class Colivara(ColiVara):
    """
    Case-insensitive alias for the `ColiVara` class.

    This class is functionally identical to `ColiVara` and exists for case-insensitivity.
    """

    pass


class colivara(ColiVara):
    """
    Case-insensitive alias for the `ColiVara` class.

    This class is functionally identical to `ColiVara` and exists for case-insensitivity.
    """

    pass


# Expose all classes for import
__all__ = ["ColiVara", "Colivara", "colivara"]