# coding: utf-8

"""
ColiVara

Colivara is a suite of services that allows you to store, search, and retrieve documents based on their visual embeddings.      It is a web-first implementation of the ColPali paper using ColQwen2 as backend model. It works exacly like RAG from the end-user standpoint - but using vision models instead of chunking and text-processing for documents.

The version of the OpenAPI document: 1.0.0
Contact: abdulhaleem@tjmlabs.com
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest

from colivara_py.models.webhook_out import WebhookOut


class TestWebhookOut(unittest.TestCase):
    """WebhookOut unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> WebhookOut:
        """Test WebhookOut
        include_optional is a boolean, when False only required
        params are included, when True both required and
        optional params are included"""
        # uncomment below to create an instance of `WebhookOut`
        """
        model = WebhookOut()
        if include_optional:
            return WebhookOut(
                app_id = '',
                endpoint_id = '',
                webhook_secret = ''
            )
        else:
            return WebhookOut(
                app_id = '',
                endpoint_id = '',
                webhook_secret = '',
        )
        """

    def testWebhookOut(self):
        """Test WebhookOut"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
