# coding: utf-8

"""
ColiVara

Colivara is a suite of services that allows you to store, search, and retrieve documents based on their visual embeddings.      It is a web-first implementation of the ColPali paper using ColQwen2 as backend model. It works exacly like RAG from the end-user standpoint - but using vision models instead of chunking and text-processing for documents.

The version of the OpenAPI document: 1.0.0
Contact: abdulhaleem@tjmlabs.com
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest


class TestLookupEnum(unittest.TestCase):
    """LookupEnum unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testLookupEnum(self):
        """Test LookupEnum"""
        # inst = LookupEnum()


if __name__ == "__main__":
    unittest.main()
