# coding: utf-8

"""
ColiVara

Colivara is a suite of services that allows you to store, search, and retrieve documents based on their visual embeddings.      It is a web-first implementation of the ColPali paper using ColQwen2 as backend model. It works exacly like RAG from the end-user standpoint - but using vision models instead of chunking and text-processing for documents.

The version of the OpenAPI document: 1.0.0
Contact: abdulhaleem@tjmlabs.com
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest

from colivara_py.models.query_out import QueryOut


class TestQueryOut(unittest.TestCase):
    """QueryOut unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> QueryOut:
        """Test QueryOut
        include_optional is a boolean, when False only required
        params are included, when True both required and
        optional params are included"""
        # uncomment below to create an instance of `QueryOut`
        """
        model = QueryOut()
        if include_optional:
            return QueryOut(
                query = '',
                results = [
                    colivara_py.models.page_out_query.PageOutQuery(
                        collection_name = '', 
                        collection_id = 56, 
                        collection_metadata = colivara_py.models.collection_metadata.collection_metadata(), 
                        document_name = '', 
                        document_id = 56, 
                        document_metadata = colivara_py.models.document_metadata.document_metadata(), 
                        page_number = 56, 
                        raw_score = 1.337, 
                        normalized_score = 1.337, 
                        img_base64 = '', )
                    ]
            )
        else:
            return QueryOut(
                query = '',
                results = [
                    colivara_py.models.page_out_query.PageOutQuery(
                        collection_name = '', 
                        collection_id = 56, 
                        collection_metadata = colivara_py.models.collection_metadata.collection_metadata(), 
                        document_name = '', 
                        document_id = 56, 
                        document_metadata = colivara_py.models.document_metadata.document_metadata(), 
                        page_number = 56, 
                        raw_score = 1.337, 
                        normalized_score = 1.337, 
                        img_base64 = '', )
                    ],
        )
        """

    def testQueryOut(self):
        """Test QueryOut"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)


if __name__ == "__main__":
    unittest.main()
