# coding: utf-8

"""
ColiVara

Colivara is a suite of services that allows you to store, search, and retrieve documents based on their visual embeddings.      It is a web-first implementation of the ColPali paper using ColQwen2 as backend model. It works exacly like RAG from the end-user standpoint - but using vision models instead of chunking and text-processing for documents.

The version of the OpenAPI document: 1.0.0
Contact: abdulhaleem@tjmlabs.com
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import unittest

from colivara_py.api.documents_api import DocumentsApi


class TestDocumentsApi(unittest.TestCase):
    """DocumentsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = DocumentsApi()

    def tearDown(self) -> None:
        pass

    def test_api_views_delete_document(self) -> None:
        """Test case for api_views_delete_document

        Delete Document
        """
        pass

    def test_api_views_get_document(self) -> None:
        """Test case for api_views_get_document

        Get Document
        """
        pass

    def test_api_views_list_documents(self) -> None:
        """Test case for api_views_list_documents

        List Documents
        """
        pass

    def test_api_views_partial_update_document(self) -> None:
        """Test case for api_views_partial_update_document

        Partial Update Document
        """
        pass

    def test_api_views_upsert_document(self) -> None:
        """Test case for api_views_upsert_document

        Upsert Document
        """
        pass


if __name__ == "__main__":
    unittest.main()
